[bits 32]

%define MULTIBOOT_MAGIC 0x1BADB002
%define MULTIBOOT_FLAGS 0x00000003

%define STACK_SIZE 0x10000

struc TSS
  .link resd 1
  .esp0 resd 1
  .ss0 resd 1
  .esp1 resd 1
  .ss1 resd 1
  .esp2 resd 1
  .ss2 resd 1
  .cr3 resd 1
  .eip resd 1
  .eflags resd 1
  .eax resd 1
  .ecx resd 1
  .edx resd 1
  .ebx resd 1
  .esp resd 1
  .ebp resd 1
  .esi resd 1
  .edi resd 1
  .es resd 1
  .cs resd 1 
  .ss resd 1
  .ds resd 1
  .fs resd 1
  .gs resd 1
  .ldtr resd 1
  .resv resw 1
  .iobp resw 1
  .sizem:
endstruc

segment .text
align 4
_multiboot_header:
  dd MULTIBOOT_MAGIC
  dd MULTIBOOT_FLAGS
  dd -(MULTIBOOT_MAGIC + MULTIBOOT_FLAGS)

extern kmain
global _start
_start:
  cld				; make sure string operations
  				; count in upwards direction
  				
  lgdt [gdtDescriptor]		; load known GDT
  jmp SEG_CODE32_KERNEL:.fixCS	; fix CS register
.fixCS:
  ; reload segment registers
  mov ax, SEG_DATA32_KERNEL
  mov ds, ax
  mov es, ax
  mov fs, ax
  mov gs, ax
  mov ss, ax
  ; fixup TSS descriptor (linker can't deal with
  ; such complex relocations gdt needs)
  mov eax, _mainTSS
  mov word [gdt.tss + 2], ax
  shr eax, 16
  mov byte [gdt.tss + 4], al
  mov byte [gdt.tss + 7], ah
      
  ; load TSS
  mov ax, SEG_MAIN_TSS | 0x0003
  ltr ax

  mov esp, stack + STACK_SIZE	; initialize known stack  
  push ebx			; multiboot info structure (see multiboot spec)
  call kmain			; start of our C code
  add esp, 4
  cli
  hlt

segment .data
gdtDescriptor:
.limit: dw gdt.end - gdt - 1
.base: dd gdt

gdt:
  dd 0x00000000
  dd 0x00000000
SEG_CODE32_KERNEL equ $ - gdt
  dd 0x0000FFFF
  dd 0x00CF9A00
SEG_DATA32_KERNEL equ $ - gdt
  dd 0x0000FFFF
  dd 0x00CF9200
SEG_MAIN_TSS equ $ - gdt
.tss:
  dd 0x00000067
  dd 0x00408900
.end:

global _mainTSS
_mainTSS: istruc TSS
  at TSS.ss0, dd SEG_DATA32_KERNEL
  at TSS.es, dd SEG_DATA32_KERNEL
  at TSS.cs, dd SEG_CODE32_KERNEL
  at TSS.ds, dd SEG_DATA32_KERNEL
  at TSS.fs, dd SEG_DATA32_KERNEL
  at TSS.gs, dd SEG_DATA32_KERNEL
  at TSS.iobp, dw 104
iend

segment .bss
stack: resb STACK_SIZE

