[bits 32]

segment .text

global cpuEnableInterrupts
cpuEnableInterrupts:
  sti
  ret

global cpuDisableInterrupts
cpuDisableInterrupts:
  pushfd
  cli
  pop eax
  shr eax, 9
  and eax, 1
  ret

global cpuRestoreInterrupts
cpuRestoreInterrupts:
  mov eax, [esp + 4]
  or eax, eax
  jz .disable
  sti
  ret
.disable:
  cli
  ret

global cpuGetEIP
cpuGetEIP:
  pop eax
  jmp eax

global cpuGetESP
cpuGetESP:
  mov eax, esp
  add eax, 4
  ret

global cpuLIDT
cpuLIDT:
  mov eax, [esp + 4]
  lidt [eax]
  ret

global cpuSystemHalt
cpuSystemHalt:
  cli
  hlt

global inb
global inw
global inl
global ind
global outb
global outw
global outl
global outd
global _insb
global _insw
global _insl
global _insd
global _outsb
global _outsw
global _outsl
global _outsd
inb:
  mov edx, [esp + 4]
  xor eax, eax
  in al, dx
  ret

inw:
  mov edx, [esp + 4]
  xor eax, eax
  in ax, dx
  ret

inl:
ind:
  mov edx, [esp + 4]
  in eax, dx
  ret

outb:
  mov edx, [esp + 4]
  mov eax, [esp + 8]
  out dx, al
  ret

outw:
  mov edx, [esp + 4]
  mov eax, [esp + 8]
  out dx, ax
  ret

outl:
outd:
  mov edx, [esp + 4]
  mov eax, [esp + 8]
  out dx, eax
  ret

_insb:
  cld
  push edi
  mov edi, [esp + 8]
  mov edx, [esp + 12]
  mov ecx, [esp + 16]
  rep insb
  pop edi
  ret

_insw:
  cld
  push edi
  mov edi, [esp + 8]
  mov edx, [esp + 12]
  mov ecx, [esp + 16]
  rep insw
  pop edi
  ret

_insl:
_insd:
  cld
  push edi
  mov edi, [esp + 8]
  mov edx, [esp + 12]
  mov ecx, [esp + 16]
  rep insd
  pop edi
  ret

_outsb:
  cld
  push esi
  mov esi, [esp + 8]
  mov edx, [esp + 12]
  mov ecx, [esp + 16]
  rep outsb
  pop esi
  ret

_outsw:
  cld
  push esi
  mov esi, [esp + 8]
  mov edx, [esp + 12]
  mov ecx, [esp + 16]
  rep outsw
  pop esi
  ret

_outsl:
_outsd:
  cld
  push esi
  mov esi, [esp + 8]
  mov edx, [esp + 12]
  mov ecx, [esp + 16]
  rep outsd
  pop esi
  ret
  
  global cpuEnterV86Int
cpuEnterV86Int:
  push ebp
  mov ebp, esp
  push ebx
  push esi
  push edi
  mov eax, [ebp + 8]
  mov ebx, [ebp + 12]
  mov ecx, [ebp + 16]
  mov edx, [ebp + 20]
  mov esi, [ebp + 24]
  mov edi, [ebp + 28]
  int 0xFD ; MUST match interrupt number registered in vm86.c
  pop edi
  pop esi
  pop ebx
  mov esp, ebp
  pop ebp
  ret

global cpuEnterV86
cpuEnterV86:
  mov ebp, esp
  push 0                ; GS
  push 0                ; FS
  push 0                ; DS
  push 0                ; ES
  push dword [ebp + 4]  ; SS
  push dword [ebp + 8]  ; ESP
  pushfd                ; EFLAGS
  or dword [esp], (1 << 17)
  push dword [ebp + 12] ; CS
  push dword [ebp + 16] ; EIP
  mov eax, dword [ebp + 20]
;  hlt
  iretd
