#ifndef CPU_H
#define CPU_H

#include <types.h>

typedef struct TSS
{
    dword PreviousTSS;
    dword ESP0, SS0;
    dword ESP1, SS1;
    dword ESP2, SS2;
    dword CR3;
    dword EIP;
    dword EFLAGS;
    dword EAX, ECX, EDX, EBX;
    dword ESP, EBP, ESI, EDI;
    dword ES, CS, SS, DS, FS, GS;
    dword LDT;
    word Trap;
    word IOMapBase;
} TSS;

void cpuEnableInterrupts();
bool cpuDisableInterrupts();		// returns true if interrupts were enabled
void cpuRestoreInterrupts(bool state);	// restores interrupt state
					// returned by cpuDisableInterrupts

void cpuSystemHalt();
uintptr_t cpuGetEIP();
uintptr_t cpuGetESP();
void cpuLIDT(void *lidt);

// io port operations
byte inb(word port);
word inw(word port);
dword inl(word port);
dword ind(word port);
void outb(word port, byte value);
void outw(word port, word value);
void outl(word port, dword value);
void outd(word port, dword value);

void _insb(void *buffer, word port, size_t n);
void _insw(void *buffer, word port, size_t n);
void _insl(void *buffer, word port, size_t n);
void _insd(void *buffer, word port, size_t n);
void _outsb(const void *buffer, word port, size_t n);
void _outsw(const void *buffer, word port, size_t n);
void _outsl(const void *buffer, word port, size_t n);
void _outsd(const void *buffer, word port, size_t n);

// vm86
void cpuEnterV86Int(int ignored, int arg1, int arg2, int arg3, int arg4, int arg6);
void cpuEnterV86(uint32_t ss, uint32_t esp, uint32_t cs, uint32_t eip, uint32_t eax);

#endif // CPU_H
