#include <cpu.h>
#include <ints.h>
#include <types.h>

#define VECTOR_COUNT 256

static InterruptHandler *handlers[VECTOR_COUNT];

void intsCommonHandler(MachineState *state)
{
    bool handled = false;
    InterruptHandler *handler = handlers[state->InterruptNumber];
    for(handled = false; !handled && handler && handler->Callback; handler = handler->Next)
        handled = handler->Callback(state, handler->Context);
    if(!handled)
    {
        _outsb("Unhandled interrupt/exception. System Halted!\n", 0xE9, 46);
        cpuSystemHalt();
    }
}

void intsRegisterHandler(int intNo, InterruptHandler *handler)
{
    if(intNo < 0 || intNo >= VECTOR_COUNT || !handler)
        return;
    bool ints = cpuDisableInterrupts();
    handler->Next = handlers[intNo];
    handlers[intNo] = handler;
    cpuRestoreInterrupts(ints);
}

void intsUnRegisterHandler(int intNo, InterruptHandler *handler)
{
    if(intNo < 0 || intNo >= VECTOR_COUNT || !handler)
        return;
    bool ints = cpuDisableInterrupts();
    for(InterruptHandler *prev = 0, *h = handlers[intNo]; h; prev = h, h = h->Next)
    {
        if(handler == h)
        {
            if(prev) prev->Next = h->Next;
            else handlers[intNo] = h->Next;
        }
    }
    cpuRestoreInterrupts(ints);
}

