#ifndef INTS_H
#define INTS_H

#include <types.h>

#pragma pack(push, 1)
typedef struct MachineState
{
    union //sESP
    {
        dword ESP;
        word SP;
    };
    struct //sSS
    {
        word SS;
        word hSS;
    };
    struct //sGS
    {
        word GS;
        word hGS;
    };
    struct //sFS
    {
        word FS;
        word hFS;
    };
    struct //sES
    {
        word ES;
        word hES;
    };
    struct //sDS
    {
        word DS;
        word hDS;
    };
    union //sEDI
    {
        dword EDI;
        word DI;
    };
    union //sESI
    {
        dword ESI;
        word SI;
    };
    union //sEBP
    {
        dword EBP;
        word BP;
    };
    union //sEBX
    {
        dword EBX;
        word BX;
        struct
        {
            byte BL;
            byte BH;
        };
    };
    union //sEDX
    {
        dword EDX;
        word DX;
        struct
        {
            byte DL;
            byte DH;
        };
    };
    union //sECX
    {
        dword ECX;
        word CX;
        struct
        {
            byte CL;
            byte CH;
        };
    };
    union //sEAX
    {
        dword EAX;
        word AX;
        struct
        {
            byte AL;
            byte AH;
        };
    };
    dword InterruptNumber;
    dword ErrorCode;

    union //sEIP
    {
        dword EIP;
        word IP;
    };
    struct //sCS
    {
        word CS;
        word hCS;
    };
    union //sEFLAGS
    {
        dword EFLAGS;
        word FLAGS;
    };
    union //sUserESP
    {
        dword UserESP;
        word UserSP;
    };
    union //sUserSS
    {
        word UserSS;
        word hUserSS;
    };
} MachineState;
#pragma pack(pop)

// returns true if interrupt was properly handled
typedef bool (*HandlerCallback)(MachineState *state, void *context);

typedef struct InterruptHandler InterruptHandler;
typedef struct InterruptHandler
{
    InterruptHandler *Next; // set to 0
    HandlerCallback Callback;
    void *Context;
} InterruptHandler;

void intsRegisterHandler(int intNo, InterruptHandler *handler);
void intsUnRegisterHandler(int intNo, InterruptHandler *handler);

#endif // INTS_H

